package com.strategyquant.plugin.Task.impl.FilterByCorrelation;

import java.util.ArrayList;
import java.util.List;

import com.strategyquant.lib.L;
import com.strategyquant.lib.SettingsMap;
import com.strategyquant.lib.constants.SQConst;
import com.strategyquant.pluginlib.annotations.Category;
import com.strategyquant.pluginlib.annotations.License;
import com.strategyquant.pluginlib.annotations.Name;
import com.strategyquant.pluginlib.annotations.ShortDesc;
import com.strategyquant.tradinglib.BadStrategyException;
import com.strategyquant.tradinglib.Databank;
import com.strategyquant.tradinglib.ProjectRunInfo;
import com.strategyquant.tradinglib.ResultsGroup;
import com.strategyquant.tradinglib.SettingsKeys;
import com.strategyquant.tradinglib.correlation.CorrelationComputer;
import com.strategyquant.tradinglib.correlation.CorrelationSettings;
import com.strategyquant.tradinglib.project.ProgressEngine;
import com.strategyquant.tradinglib.project.ProjectEngine;
import com.strategyquant.tradinglib.project.ProjectGlobalLog;
import com.strategyquant.tradinglib.project.SQProject;
import com.strategyquant.tradinglib.results.SpecialValues;
import com.strategyquant.tradinglib.taskImpl.AbstractTask;
import com.strategyquant.tradinglib.taskImpl.ISQTask;

import net.xeoh.plugins.base.annotations.PluginImplementation;
import net.xeoh.plugins.base.annotations.meta.Author;

@Author(name = "Tamas Takacs")
@Name(name = "Filter by correlation")
@Category(name = "Custom Task")
@License(text = "")
@ShortDesc(text = "")
@PluginImplementation
public class TaskFilterByCorrelation extends AbstractTask {
	
	private CorrelationComputer correlationComputer = new CorrelationComputer();
	
	private Databank databankSource = null;
	private Databank databankTarget = null;
	private Databank databankExisting = null;

	private CorrelationSettings correlation = null;

	private int totalResults;
	private int totalCopied = 0;
	
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	
	public TaskFilterByCorrelation() throws Exception {
		super(null, null);
	}
	
	//------------------------------------------------------------------------
	
	public TaskFilterByCorrelation(String projectName, ProgressEngine progressEngine) throws Exception {
		super(projectName, progressEngine);
	}
	
	//------------------------------------------------------------------------
	
	@Override
	public String getType() {
		return "FilterByCorrelation";
	}
	
	//------------------------------------------------------------------------
	
	@Override
	public String getName() {
		return L.tsq("Filter by correlation");
	}

	//------------------------------------------------------------------------
	
	@Override
	public ISQTask clone(String projectName, ProgressEngine progressEngine) throws Exception {
		TaskFilterByCorrelation task = new TaskFilterByCorrelation(projectName, progressEngine);
		task.settingTabPlugins = this.settingTabPlugins;
		
		return task;
	}
	
	
	//------------------------------------------------------------------------

	private void loadSettings() {				
		this.databankSource = (Databank) settingsData.getParams().get(SettingsKeys.DatabankSource);
		this.databankTarget = (Databank) settingsData.getParams().get(SettingsKeys.DatabankTarget);
		this.databankExisting = (Databank) settingsData.getParams().get("DatabankExisting");
		
		this.correlation = (CorrelationSettings) settingsData.getParams().get(SettingsKeys.Correlation);
	}
	
	//------------------------------------------------------------------------
	
	@Override
	public void start() throws Exception {
		loadSettings();
		
		this.project = ProjectEngine.get(projectName);		
		
		progressEngine.setLogPrefix(taskLogPrefix);
		progressEngine.printToLog(L.t("Starting strategies filtering..."));
		progressEngine.start();
			
		ResultsGroup strategyCloned;
		ArrayList<ResultsGroup> results = databankSource.getRecords();

		this.totalCopied = 0;
		this.totalResults = results.size();

		for(int i=0; i<results.size(); i++) {
			ResultsGroup strategy = results.get(i);
			String strategyName = strategy.getName();
			
			try {
				String dismissalMessage = checkCorrelation(strategy);
				if(dismissalMessage != null) {
					progressEngine.printToLogDebug(L.t("%s skipped, %s", strategyName, dismissalMessage));
					increaseGlobalStats(false);
					
					continue;
				}

				//copy from source to target
				strategyCloned = strategy.clone();
				databankTarget.add(strategyCloned, true);
				
				progressEngine.printToLogDebug(L.t("%s copied, %s", strategyName, String.format("correlation < %.2f", correlation.maxCorrelation)));
				
				this.totalCopied++;
				
				increaseGlobalStats(true);
				
				progressEngine.checkPaused();
				
				if(progressEngine.isStopped()) {
					break;
				}
			}
			catch (OutOfMemoryError e) {
				project.onMemoryError(e);
				break;
			}
		}
		
		databankTarget.updateBestResults();
		
		progressEngine.finish();
	}
	
	//------------------------------------------------------------------------
	
	private void increaseGlobalStats(boolean passed) {
		project.increaseGlobalStats(getCustomName().hashCode(), L.t("Input"), 1, "Copied", passed ? 1 : 0);
	}
	
	//------------------------------------------------------------------------
	
	private String checkCorrelation(ResultsGroup strategy) {
		String dismissalMessage = null;
		
		ArrayList<ResultsGroup> results = databankExisting.getRecords();
		for(int i=0; i<results.size(); i++) {
			ResultsGroup strategyExisting = results.get(i);
			String combination = String.format("%s - %s", strategy.getName(), strategyExisting.getName());

			double corr = correlationComputer.computeCorrelation(strategy, strategyExisting, correlation.correlationPeriod, correlation.correlationType, correlation.addEmptyPeriods);

			if(corr > correlation.maxCorrelation || (!correlation.allowNegativeCorrelation && corr < 0)) {					
				if(corr < 0) {
					return L.t("%s correlation %.2f < 0, negative correlation not allowed", combination, corr);
				} else {
					return L.t("%s correlation %.2f > %.2f", combination, corr, correlation.maxCorrelation);
				}
			}
		}
		
		return dismissalMessage;
	}
	
	//------------------------------------------------------------------------
	
	@Override
	protected int getRunningStatus() {
		return 0;
	}

	//------------------------------------------------------------------------
	
	@Override
	public String getPluginFolderName() {
		return "TaskFilterByCorrelation";
	}
	
	//------------------------------------------------------------------------
	
	@Override
	public int getPreferredPosition() {
		return 1;
	}

	//------------------------------------------------------------------------

	@Override
	public String[] getSettings() {
		return new String[] { 
			"FilterByCorrelation" 
		};
	}

	//------------------------------------------------------------------------

	@Override
	protected Databank[] getUsedDatabanks() {
		return new Databank[]{ databankSource, databankTarget, databankExisting };
	}

	//------------------------------------------------------------------------

	@Override
	protected Databank getOutputDatabank() {
		return databankTarget;
	}
}


